unit ClientDataSetQuery;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBTables, DBClient, BdeProv{$IFDEF VER120}, Provider{$ENDIF};


type
  TClientDataSetQuery = class(TClientDataSet)
  private
    { Private declarations }
    FProvider: TProvider;
    FQuery: TQuery;
    FUpdateSQL: TUpdateSQL;
    FBeforeUpdateRecord: TBeforeUpdateRecordEvent;
    function GetOnUpdateError: TResolverErrorEvent;
    procedure SetOnUpdateError(Event: TResolverErrorEvent);
    function GetAfterUpdateRecord: TAfterUpdateRecordEvent;
    procedure SetAfterUpdateRecord(Event: TAfterUpdateRecordEvent);
    function GetSQLIndex(Index: Integer): TStrings;
    procedure SetSQLIndex(Index: Integer; Value: TStrings);
    function GetParamsIndex(Index: Integer): TParams;
    procedure SetParamsIndex(Index: Integer; Value: TParams);
    function GetSQL: TStrings;
    procedure SetSQL(Value: TStrings);
    function GetParams: TParams;
    procedure SetParams(Value: TParams);
    function GetDatabaseName: String;
    procedure SetDatabaseName(const DatabaseName: String);
    function GetSessionName: String;
    procedure SetSessionName(const SessionName: String);
    function GetProviderName: String;
    procedure SetProviderName(const Value: String);
    procedure ProviderBeforeUpdateRecord(Sender: TObject;
      SourceDS: TDataSet; DeltaDS: TClientDataSet; UpdateKind: TUpdateKind;
      var Applied: Boolean);
  protected
    { Protected declarations }
    procedure InternalClose; override;
    procedure Loaded; override;
  public
    { Public declarations }
    procedure BindParameters(Query: TQuery; DeltaData: TDataSet);
    constructor Create(AOwner: TComponent); override;
    property Query: TQuery read FQuery;
    property UpdateSQL: TUpdateSQL read FUpdateSQL;
    property UpdateProvider: TProvider read FProvider;
  published
    { Published declarations }
    property DatabaseName: String read GetDatabaseName write SetDatabaseName;
    property SessionName: String read GetSessionName write SetSessionName;
    property SQL: TStrings read GetSQL write SetSQL;
    property Params: TParams read GetParams write SetParams;
    property ModifySQL: TStrings index 0 read GetSQLIndex write SetSQLIndex;
    property InsertSQL: TStrings index 1 read GetSQLIndex write SetSQLIndex;
    property DeleteSQL: TStrings index 2 read GetSQLIndex write SetSQLIndex;
    property ProviderName: String read GetProviderName write SetProviderName;
    {property ModifyParams: TParams index 0 read GetParamsIndex write SetParamsIndex;
    property InsertParams: TParams index 1 read GetParamsIndex write SetParamsIndex;
    property DeleteParams: TParams index 2 read GetParamsIndex write SetParamsIndex;}
    property BeforeUpdateRecord: TBeforeUpdateRecordEvent read FBeforeUpdateRecord write FBeforeUpdateRecord;
    property AfterUpdateRecord: TAfterUpdateRecordEvent read GetAfterUpdateRecord write SetAfterUpdateRecord;
    property OnUpdateError: TResolverErrorEvent read GetOnUpdateError write SetOnUpdateError;
  end;

implementation

constructor TClientDataSetQuery.Create(AOwner: TComponent);
begin
     FProvider:=TProvider.Create(Self);
     FQuery:=TQuery.Create(Self);
     FUpdateSQL:=TUpdateSQL.Create(Self);
     FQuery.UpdateObject:=FUpdateSQL;
     FProvider.DataSet:=FQuery;
     FProvider.BeforeUpdateRecord:=ProviderBeforeUpdateRecord;
     inherited Create(AOwner);
     Self.Provider:=FProvider.Provider;
end;

procedure TClientDataSetQuery.Loaded;
begin
     inherited Loaded;
     FProvider.Name:=Name;
end;

(* BindParameters - Source code copied from TUpdateSQLProvider *)
procedure TClientDataSetQuery.BindParameters(Query: TQuery; DeltaData: TDataSet);
var
  I: Integer;
  Old: Boolean;
  Param: TParam;
  PName: string;
  Field: TField;
  Value: Variant;
begin
  with Query do
    for I := 0 to Params.Count - 1 do
    begin
      Param := Params[I];
      PName := Param.Name;
      Old := CompareText(Copy(PName, 1, 4), 'OLD_') = 0;
      if Old then System.Delete(PName, 1, 4);
      Field := DeltaData.FindField(PName);
      if not Assigned(Field) then Continue;
      if Old then Param.AssignFieldValue(Field, Field.OldValue) else
      begin
        Value := Field.NewValue;
        { When we have no new value, use the old value }
        if VarIsEmpty(Value) then Value := Field.OldValue;
        Param.AssignFieldValue(Field, Value);
      end;
    end;
end;

procedure TClientDataSetQuery.ProviderBeforeUpdateRecord(Sender: TObject;
  SourceDS: TDataSet; DeltaDS: TClientDataSet; UpdateKind: TUpdateKind;
  var Applied: Boolean);
begin
     if Assigned(FBeforeUpdateRecord) then
     begin
       FBeforeUpdateRecord(Sender,SourceDS,DeltaDS,UpdateKind,Applied);
       if Applied then
          Exit;
     end;
     while not DeltaDS.EOF do
     begin
          BindParameters(FUpdateSQL.Query[UpdateKind], DeltaDS);
          FUpdateSQL.Query[UpdateKind].ExecSQL;
          DeltaDS.Next;
     end;
     Applied:=True;
end;

function TClientDataSetQuery.GetSQLIndex(Index: Integer): TStrings;
begin
  Result := FUpdateSQL.SQL[TUpdateKind(Index)];
end;

procedure TClientDataSetQuery.SetSQLIndex(Index: Integer; Value: TStrings);
begin
  FUpdateSQL.SQL[TUpdateKind(Index)] := Value;
end;

function TClientDataSetQuery.GetParamsIndex(Index: Integer): TParams;
begin
  Result := FUpdateSQL.Query[TUpdateKind(Index)].Params;
end;

procedure TClientDataSetQuery.SetParamsIndex(Index: Integer; Value: TParams);
begin
  FUpdateSQL.Query[TUpdateKind(Index)].Params := Value;
end;

function TClientDataSetQuery.GetSQL: TStrings;
begin
     Result:=FQuery.SQL;
end;

procedure TClientDataSetQuery.SetSQL(Value: TStrings);
begin
     FQuery.SQL:=Value;
end;

function TClientDataSetQuery.GetParams: TParams;
begin
     Result:=FQuery.Params;
end;

procedure TClientDataSetQuery.SetParams(Value: TParams);
begin
     FQuery.Params:=Value;
end;

function TClientDataSetQuery.GetDatabaseName: String;
begin
     Result:=FQuery.DatabaseName;
end;

procedure TClientDataSetQuery.SetDatabaseName(const DatabaseName: String);
begin
     FQuery.DatabaseName:=DatabaseName;
end;

function TClientDataSetQuery.GetSessionName: String;
begin
     Result:=FQuery.SessionName;
end;

procedure TClientDataSetQuery.SetSessionName(const SessionName: String);
begin
     FQuery.SessionName:=SessionName;
end;

function TClientDataSetQuery.GetOnUpdateError: TResolverErrorEvent;
begin
     Result:=FProvider.OnUpdateError;
end;

procedure TClientDataSetQuery.SetOnUpdateError(Event: TResolverErrorEvent);
begin
     FProvider.OnUpdateError:=Event;
end;

function TClientDataSetQuery.GetAfterUpdateRecord: TAfterUpdateRecordEvent;
begin
     Result:=FProvider.AfterUpdateRecord;
end;

procedure TClientDataSetQuery.SetAfterUpdateRecord(Event: TAfterUpdateRecordEvent);
begin
     FProvider.AfterUpdateRecord:=Event;
end;

procedure TClientDataSetQuery.InternalClose;
begin
     inherited InternalClose;
     FQuery.Active:=False;
end;

function TClientDataSetQuery.GetProviderName;
begin
     Result:=inherited ProviderName;
end;

procedure TClientDataSetQuery.SetProviderName(const Value: String);
begin
     if Value='' then
        Self.Provider:=FProvider.Provider
     else
         inherited ProviderName:=Value;
end;

end.
